c
c ------------------------------------------------------------------
c
      subroutine flow(ql,qr,sn,st,meqn,iflip,iflow,mw0)
      implicit double precision (a-h,o-z)
      dimension work(20),ql(20),qr(20),s(3)
c
c     # compute flow velocity
c
c     # iflip = 1 indicates the normal velocity is in
c               qr(i,2) and the tangential in qr(i,3)
c             = 0 indicates the reverse.
c
      inorm = 3-iflip
      itan  = 2+iflip
c
      do m=1,meqn
         work(m) = ql(m)
         enddo
      work(2) = ql(inorm)
      work(3) = ql(itan)
      call ctoprm(work,rhol,ul,vl,rhoe1,rhok1,pl,gam1,gru1,gam10,
     &            alp1,bet1,pn1,psi1)
c
      do m=1,meqn
         work(m) = qr(m)
         enddo
      work(2) = qr(inorm)
      work(3) = qr(itan)
      call ctoprm(work,rhor,ur,vr,rhoe2,rhok2,pr,gam2,gru2,gam20,
     &            alp2,bet2,pn2,psi2)
c
      go to (100,200) iflow
c
 100  continue
c     # two-shock approximation
c     #  ref. Colella & Glaz: JCP 59, 264-286 (1985)
c
      csql = gam1*rhol*(pl+pn1)+
     &       (gam1-2.d0)*alp1*rhol**3
      csqr = gam2*rhor*(pr+pn2)+
     &       (gam2-2.d0)*alp2*rhor**3
c
      if ((csql .le. 0.d0) .or. (csqr .le. 0.d0)) then
          write(6,*) '> Error in flow:'
          write(6,*) 'rhol=',rhol,', pl=',pl,pn1,gam1
          write(6,*) 'rhor=',rhor,', pr=',pr,pn2,gam2
          stop
      endif
c
c     # secant iteration version
c
      call rpsect(pstar,
     &            rhol,ul,pl,gam1,gam10,alp1,bet1,pn1,csql,
     &            rhor,ur,pr,gam2,gam20,alp2,bet2,pn2,csqr,
     &            nblow)
      if (nblow .eq. 1) return
c
      call wsqge(wsql,dummy,rho1,pstar,rhol,pl,csql,gam10,
     &           alp1,bet1,pn1)
      wl = dsqrt(wsql)
c
      call wsqge(wsqr,dummy,rho2,pstar,rhor,pr,csqr,gam20,
     &           alp2,bet2,pn2)
      wr = dsqrt(wsqr)
c
c     # mid-state velocity
      ustar = (pl-pr+ul*wl+ur*wr)/(wl+wr)
      if (dabs(ustar) .lt. 1.d-12) ustar = 0.d0
c
      s(1) = ustar-wl/rho1
      s(2) = ustar
      s(3) = ustar+wr/rho2
c
      sn = s(mw0)
      st = .5d0*(vl+vr)
      return
c
 200  continue
c     # Roe's approximate Riemann solver
c
      rlh = dsqrt(rhol)
      rrh = dsqrt(rhor)
      rhsq2 = rlh+rrh
      rho = rlh*rrh
      u = (ul*rlh+ur*rrh)/rhsq2     
      v = (vl*rlh+vr*rrh)/rhsq2     
      enth = ((ql(4)+pl)/rlh+(qr(4)+pr)/rrh)/rhsq2    
c
      grue = rhsq2/(rlh/gru1+rrh/gru2)
c
c     # pressure
      xeng = ((pl/gru1*rlh)+(pr/gru2*rrh))/rhsq2
      p = xeng*grue
c
      phia = (ql(6)/rlh+qr(6)/rrh)/rhsq2
      phib = (pn1*rlh+pn2*rrh)/rhsq2*phia
      alp = (alp1*rlh+alp2*rrh)/rhsq2
      phic = ((2.d0/grue)-2.d0-phia*rho)*alp*rho
      engk = 0.5d0*(u*u+v*v)
c
c     # sound speed
      csq0 = grue*(enth-engk+p*phia-phib-phic)
      c    = dsqrt(csq0)
c
      s(1) = u-c
      s(2) = u
      s(3) = u+c
c
      sn = s(mw0)
      st = v
      return
c
 1001 format(6(f12.6,x))
      end
