c
c ------------------------------------------------------------------
c
      subroutine rpjmp(ql,qr,s,qjump,work,meqn,iflip,iriem,nblow)
      implicit double precision (a-h,o-z)
      dimension work(meqn)
      dimension ql(20),qr(20),s(3),qjump(20,3),qmid(20)
c
c     # scalar Riemann problem solver for Euler equations of gas dynamics
c     # with van der waals type EOS
c
c     # iflip = 1 indicates the normal velocity is in
c               qr(2) and the tangential in qr(3)
c             = 0 indicates the reverse.
c
      nblow = 0
      inorm = 3-iflip
      itan  = 2+iflip
c
      do m=1,meqn
         work(m) = ql(m)
         enddo
      work(2) = ql(inorm)
      work(3) = ql(itan)
      call ctoprm(work,rhol,ul,vl,rhoe1,rhok1,pl,gam1,gru1,gam10,
     &            alp1,bet1,pn1,psi1)
c
      do m=1,meqn
         work(m) = qr(m)
         enddo
      work(2) = qr(inorm)
      work(3) = qr(itan)
      call ctoprm(work,rhor,ur,vr,rhoe2,rhok2,pr,gam2,gru2,gam20,
     &            alp2,bet2,pn2,psi2)
c
      go to (100,100) iriem
c
 100  continue
c     # two-shock approximation
c     #  ref. Colella & Glaz: JCP 59, 264-286 (1985)
c
      csql = gam1*rhol*(pl+pn1)+
     &       (gam1-2.d0)*alp1*rhol**3
      csqr = gam2*rhor*(pr+pn2)+
     &       (gam2-2.d0)*alp2*rhor**3 
c
      if ((csql .le. 0.d0) .or. (csqr .le. 0.d0)) then
          write(6,*) '> Error in rpjmp:'
          write(6,*) 'rhol=',rhol,', pl=',pl,pn1,gam1
          write(6,*) 'rhor=',rhor,', pr=',pr,pn2,gam2
          stop 
      endif 
c
c     # secant iteration version
      call rpsect(pstar,
     &            rhol,ul,pl,gam1,gam10,alp1,bet1,pn1,csql,
     &            rhor,ur,pr,gam2,gam20,alp2,bet2,pn2,csqr,
     &            nblow)
      if (nblow .eq. 1) return
c
c     # compute jumps across each wave   
      call wsqge(wsql,dummy,rho1,pstar,rhol,pl,csql,gam10,
     &           alp1,bet1,pn1)
      wl = dsqrt(wsql)
c
      call wsqge(wsqr,dummy,rho2,pstar,rhor,pr,csqr,gam20,
     &           alp2,bet2,pn2)
      wr = dsqrt(wsqr)
c
c     # compute mid-state velocity
      ustar = (pl-pr+ul*wl+ur*wr)/(wl+wr)
      if (dabs(ustar) .lt. 1.d-12) ustar = 0.d0
c
c     # left wave:
c     ============
      s(1) = ustar-wl/rho1
c
      call prmtoc(work,rho1,ustar,vl,pstar,gam10,
     &            alp1,bet1,pn1,psi1)
      do m=1,meqn
         qjump(m,1) = work(m)-ql(m)
         enddo
      qjump(inorm,1) = work(2)-ql(inorm)
      qjump(itan,1)  = work(3)-ql(itan)
c
c     # right wave:
c     =============
      s(3) = ustar+wr/rho2
c
      call prmtoc(qmid,rho2,ustar,vr,pstar,gam20,
     &            alp2,bet2,pn2,psi2)
      do m=1,meqn
         qjump(m,3) = qr(m)-qmid(m)
         enddo
      qjump(inorm,3) = qr(inorm)-qmid(2)
      qjump(itan,3)  = qr(itan)-qmid(3)
c
c     # slip line:
c     ============
      s(2) = ustar
c
      do m=1,meqn
         qjump(m,2) = qmid(m)-work(m)
         enddo
      qjump(inorm,2) = qmid(2)-work(2)
      qjump(itan,2)  = qmid(3)-work(3)
      return
c
      end
