c
c -------------------------------------------------------------------
c
      subroutine rprot(ql,qr,s,qjump,work,alf,beta,meqn,mwave,ltrack,
     &                 iriem)
      implicit double precision (a-h,o-z)
      common /tmp/ stan0,stan1,stan2
      dimension work(meqn),qjump(20,3),s(3)
      dimension ql(20),qr(20),qmid(20),qlin(20),qrin(20)
      logical ltrack(3)
c
c     # rotated Riemann problem solver for Euler equations of gas dynamics
c     # in (alf,beta) direction
c     # version of van der waals EOS
c
c     # Given states ql, qr, and rotated direcction
c     # resolve into m=meqn discontinuities
c     # qjump(*,1),...,qjump(*,m) propagating at speeds s(1),...,s(m).
c
      do m=1,meqn
         qlin(m) = ql(m)
         qrin(m) = qr(m)
         enddo
c
      qlin(2) = alf*ql(2)+beta*ql(3)
      qrin(2) = alf*qr(2)+beta*qr(3)
      qlin(3) = beta*ql(2)-alf*ql(3)
      qrin(3) = beta*qr(2)-alf*qr(3)
c
      call ctoprm(qlin,rhol,ul,vl,rhoe1,rhok1,pl,gam1,gru1,
     &            gam10,alp1,bet1,pn1,psi1)    
      call ctoprm(qrin,rhor,ur,vr,rhoe2,rhok2,pr,gam2,gru2,
     &            gam20,alp2,bet2,pn2,psi2)
c
      go to (100) iriem
c
 100  continue
c     # two-shock approximation
c     #  ref. Colella & Glaz: JCP 59, 264-286 (1985) 
c
      stan0 = .5d0*(vl+vr)
c
      csql = gam1*rhol*(pl+pn1)+
     &       (gam1-2.d0)*alp1*rhol**3
      csqr = gam2*rhor*(pr+pn2)+
     &       (gam2-2.d0)*alp2*rhor**3 
c
      if ((csql .le. 0.d0) .or. (csqr .le. 0.d0)) then
          write(6,*) '> Error in rprot:'
          write(6,*) 'rhol=',rhol,', pl=',pl,pn1,gam1
          write(6,*) 'rhor=',rhor,', pr=',pr,pn2,gam2
          stop
      endif 
c
c     # secant iteration version
      call rpsect(pstar,
     &            rhol,ul,pl,gam1,gam10,alp1,bet1,pn1,csql,
     &            rhor,ur,pr,gam2,gam20,alp2,bet2,pn2,csqr,
     &            nblow)
      if (nblow .eq. 1) stop
c
c     # compute jumps across each wave
      call wsqge(wsql,dummy,rho1,pstar,rhol,pl,csql,gam10,
     &           alp1,bet1,pn1)
      wl = dsqrt(wsql)
c
      call wsqge(wsqr,dummy,rho2,pstar,rhor,pr,csqr,gam20,
     &           alp2,bet2,pn2)
      wr = dsqrt(wsqr)
c
c     # compute mid-state velocity
      ustar = (pl-pr+ul*wl+ur*wr)/(wl+wr)  
      if (dabs(ustar) .lt. 1.d-12) ustar = 0.d0
c
c     # left wave:
c     ============
      ltrack(1) = pstar .gt. pl
      s(1) = ustar-wl/rho1
c
      call prmtoc(work,rho1,ustar,vl,pstar,gam10,
     &            alp1,bet1,pn1,psi1)
      do m=1,meqn
         qjump(m,1) = work(m)-qlin(m)
         enddo
c
c     # right wave:
c     =============
      ltrack(3) = pstar .gt. pr
      s(3) = ustar+wr/rho2
c
      call prmtoc(qmid,rho2,ustar,vr,pstar,gam20,
     &            alp2,bet2,pn2,psi2) 
      do m=1,meqn
         qjump(m,3) = qrin(m)-qmid(m)
         enddo
c
c     # slip line:
c     ============
      ltrack(2) = .true.
      s(2) = ustar
c
      do m=1,meqn
         qjump(m,2) = qmid(m)-work(m)
         enddo
      go to 900
c
 900  continue
c     # rotate back to the x- and y- coordinates
      do mw=1,mwave
	 qjnorm = alf*qjump(2,mw)+beta*qjump(3,mw)
	 qjump(3,mw) = beta*qjump(2,mw)-alf*qjump(3,mw)
	 qjump(2,mw) = qjnorm
         enddo
      return
c
      end
