c
c ------------------------------------------------------------------
c
      subroutine vflow(ql,qr,s,work,imin,imax,mmax,meqn,mwave,
     &                 iflip,iflow,mw0)
      implicit double precision (a-h,o-z)
      parameter(nxtot=610,nytot=610,isize=max(nxtot,nytot))
      dimension ql(mmax,meqn),qr(mmax,meqn),work(meqn)
      dimension s(mmax,mwave)
      dimension rhol(isize),ul(isize),vl(isize),pl(isize)
      dimension rhor(isize),ur(isize),vr(isize),pr(isize)
      dimension gam1(isize),gam2(isize),gam10(isize),gam20(isize)
      dimension gru1(isize),gru2(isize),alp1(isize),alp2(isize)
      dimension pn1(isize),pn2(isize),bet1(isize),bet2(isize)
      dimension psi1(isize),psi2(isize),csql(isize),csqr(isize)
      dimension pstar(isize),s0(3)
c
c     # compute flow velocity
c
c     # iflip = 1 indicates the normal velocity is in
c               qr(i,2) and the tangential in qr(i,3)
c             = 0 indicates the reverse.
c
      inorm = 3-iflip
      itan  = 2+iflip
c
      do i=imin,imax
         do m=1,meqn
            work(m) = ql(i,m)
            enddo
         work(2) = ql(i,inorm)
         work(3) = ql(i,itan)
         call ctoprm(work,rhol(i),ul(i),vl(i),dum0,dum1,pl(i),
     &               gam1(i),gru1(i),gam10(i),alp1(i),bet1(i),
     &               pn1(i),psi1(i))
c
         do m=1,meqn
            work(m) = qr(i,m)
            enddo
         work(2) = qr(i,inorm)
         work(3) = qr(i,itan)
         call ctoprm(work,rhor(i),ur(i),vr(i),dum0,dum1,pr(i),
     &               gam2(i),gru2(i),gam20(i),alp2(i),bet2(i),
     &               pn2(i),psi2(i))
         enddo 
c
      go to (100,200) iflow
c
 100  continue
c     # two-shock approximation
c     # ref. Colella & Glaz: JCP 59, 264-286 (1985)
c
      do i=imin,imax
         csql(i) = gam1(i)*rhol(i)*(pl(i)+pn1(i))+
     &             (gam1(i)-2.d0)*alp1(i)*rhol(i)**3
         csqr(i) = gam2(i)*rhor(i)*(pr(i)+pn2(i))+
     &             (gam2(i)-2.d0)*alp2(i)*rhor(i)**3 
c
         if((csql(i) .le. 0.d0) .or. (csqr(i) .le. 0.d0)) then
            write(6,*) '> Error in vflow:',
     &                 ' i=',i
            write(6,*) 'rhol=',rhol(i),', pl=',pl(i),pn1(i),
     &                         gam1(i)
            write(6,*) 'rhor=',rhor(i),', pr=',pr(i),pn2(i),
     &                         gam2(i)
            stop
         endif
         enddo 
c
c     # secant iteration version
      do i=imin,imax
         call rpsect(pstar(i),
     &               rhol(i),ul(i),pl(i),gam1(i),
     &               gam10(i),alp1(i),bet1(i),pn1(i),csql(i),
     &               rhor(i),ur(i),pr(i),gam2(i),gam20(i),
     &               alp2(i),bet2(i),pn2(i),csqr(i),nblow)
         if (nblow .eq. 1) stop
         enddo 
c
      do i=imin,imax
         call wsqge(wsql,dummy,rho1,pstar(i),rhol(i),pl(i),csql(i),
     &              gam10(i),alp1(i),bet1(i),pn1(i))
         wl = dsqrt(wsql)
c
         call wsqge(wsqr,dummy,rho2,pstar(i),rhor(i),pr(i),csqr(i),
     &              gam20(i),alp2(i),bet2(i),pn2(i))
         wr = dsqrt(wsqr) 
c
c        # mid-state velocity
         ustar = (pl(i)-pr(i)+ul(i)*wl+ur(i)*wr)/(wl+wr)
         if (dabs(ustar) .lt. 1.d-12) ustar = 0.d0
c
         s0(1) = ustar-wl/rho1
         s0(2) = ustar
         s0(3) = ustar+wr/rho2
c
         s(i,1) = s0(mw0)
         s(i,2) = .5d0*(vl(i)+vr(i))
         enddo
      return
c
 200  continue
c     # Roe's approximate Riemann solver
c
      do i=imin,imax
         rlh = dsqrt(rhol(i))
         rrh = dsqrt(rhor(i))
         rhsq2 = rlh+rrh
         rho = rlh*rrh
         u = (ul(i)*rlh+ur(i)*rrh)/rhsq2     
         v = (vl(i)*rlh+vr(i)*rrh)/rhsq2     
         enth = ((ql(i,4)+pl(i))/rlh+(qr(i,4)+pr(i))/rrh)/rhsq2    
c
         grue = rhsq2/(rlh/gru1(i)+rrh/gru2(i))
c
c        # pressure
         xeng = ((pl(i)/gru1(i)*rlh)+(pr(i)/gru2(i)*rrh))/rhsq2
         p = xeng*grue
c
         phia = (ql(i,6)/rlh+qr(i,6)/rrh)/rhsq2
         phib = (pn1(i)*rlh+pn2(i)*rrh)/rhsq2*phia
         alp = (alp1(i)*rlh+alp2(i)*rrh)/rhsq2
         phic = ((2.d0/grue)-2.d0-phia*rho)*alp*rho
         engk = 0.5d0*(u*u+v*v)
c
c        # sound speed
         csq0 = grue*(enth-engk+p*phia-phib-phic)
         c    = dsqrt(csq0)      
c
         s0(1) = u-c
         s0(2) = u
         s0(3) = u+c
c
         s(i,1) = s0(mw0)
         s(i,2) = v
         enddo
      return
c
 1001 format(6(f12.6,x))
      end
