c
c ------------------------------------------------------------------
c
      subroutine vofrc(ix,jy,vof,alf,beta,xlow,ylow,hx,hy,area0,
     &                 x1,y1,x2,y2)
      implicit double precision (a-h,o-z)
      parameter(eps=1.d-10,itmax=50)
      dimension p1(20,2),p2(20,2)
      data idebug/0/
c
c     # interface reconstruction
c     # given vof (volume of fluid) of the cell & the approximate
c     # normal direction of the interface
c     # locate the line segment that preserves the vof
c
c     # method: a bisection approach
c
      if (idebug .eq. 1) write(6,1001) ix,jy,vof,alf,beta
c
      xi = xlow+dfloat(ix-1)*hx
      yj = ylow+dfloat(jy-1)*hy
      xc = xi+0.5d0*hx
      yc = yj+0.5d0*hy
c
      p1(1,1) = xi
      p1(1,2) = yj
      p1(2,1) = xi
      p1(2,2) = yj+hy
      p1(3,1) = xi+hx
      p1(3,2) = yj+hy
      p1(4,1) = xi+hx
      p1(4,2) = yj
      p1(5,1) = xi
      p1(5,2) = yj
      p1(6,1) = -99.d0
      p1(6,2) = -99.d0 
c
c     # initial guess: half space sweeping
c     # look for sign change of "z" (the difference between the region cut
c     # by the half space and the underlying vof)
      xc0 = xc
      yc0 = yc
      call vofex(vof,alf,beta,p1,xc0,yc0,z0,p2,xc1,yc1,area,area0)
      if (z0 .lt. 0.d0) then
c         # half space advances forward
          xc1 = (area0*xc-area*xc1)/(area0-area)
          yc1 = (area0*yc-area*yc1)/(area0-area)
      endif
c
      do 10 iter=1,itmax
         call vofex(vof,alf,beta,p1,xc1,yc1,z1,p2,xc3,yc3,area,area0)
         if (z0*z1 .le. 0.d0) go to 15
         xc0 = xc1
         yc0 = yc1
         z0  = z1
         if (z1 .ge. 0.d0) then
c            # half space advances backward
             xc1 = xc3
             yc1 = yc3
         else
c            # half space advances forward
             xc1 = (area0*xc-area*xc3)/(area0-area)
             yc1 = (area0*yc-area*yc3)/(area0-area)
         endif
 10      continue
      write(6,*) 'ERROR in vofrc: z1=',z1
      write(6,1001) ix,jy,vof,alf,beta
      stop
c
 15   continue
c     # iterate to locate the final interface position
      do 18 iter=1,itmax
         xc3 = .5d0*(xc0+xc1)
         yc3 = .5d0*(yc0+yc1)
         call vofex(vof,alf,beta,p1,xc3,yc3,z,p2,xc,yc,area,area0)
c
         if (dabs(z) .le. eps) go to 20
c
         if (z*z0 .le. 0.d0) then
             xc1 = xc3
             yc1 = yc3
         else
             xc0 = xc3
             yc0 = yc3
             z0  = z
         endif
 18      continue
      write(6,*) 'ERROR in vofrc: z=',z
      stop
c
 20   continue
c     # recover linear segment
c
      do i=1,20
         if (p2(i+1,1) .eq. -99.d0) go to 29
         x1 = p2(i,1)
         y1 = p2(i,2)
         x2 = p2(i+1,1)
         y2 = p2(i+1,2)
         if (dabs(area2(x1,y1,x2,y2,xc3,yc3)) .le. 1.d-12)
     &       return
         enddo
c
 29   continue
      write(6,*) 'ERROR in vofrc: fail to find interface' 
      stop
c
 1001 format('Entered vofrc: ix=',i3', jy=',i3,', vof=',e18.10,
     &       ', alf=',f10.6,', beta=',f10.6)
      end
