c
c ------------------------------------------------------------------
c
      subroutine vrpjmp(ql,qr,s,qjump,work,imin,imax,mmax,meqn,
     &                  mwave,iflip,iriem)
      implicit double precision (a-h,o-z)
      parameter(nxtot=610,nytot=610,isize=max(nxtot,nytot))
      dimension ql(mmax,meqn),qr(mmax,meqn),work(meqn)
      dimension s(mmax,mwave),qjump(mmax,meqn,mwave)
      dimension rhol(isize),ul(isize),vl(isize),pl(isize)
      dimension rhor(isize),ur(isize),vr(isize),pr(isize)
      dimension gam1(isize),gam2(isize),gam10(isize),gam20(isize)
      dimension alp1(isize),alp2(isize)
      dimension pn1(isize),pn2(isize),bet1(isize),bet2(isize)
      dimension psi1(isize),psi2(isize),csql(isize),csqr(isize)
      dimension pstar(isize)
      dimension qmid(20)
c
c     # vector Riemann problem solver for Euler equations of gas dynamics
c     # with van der waals type EOS
c
c     # iflip = 1 indicates the normal velocity is in
c               qr(i,2) and the tangential in qr(i,3)
c             = 0 indicates the reverse.
c
      inorm = 3-iflip
      itan  = 2+iflip
c
      do i=imin,imax
         do m=1,meqn
	    work(m) = ql(i,m)
            enddo
	 work(2) = ql(i,inorm)
	 work(3) = ql(i,itan)
         call ctoprm(work,rhol(i),ul(i),vl(i),dum0,dum1,pl(i),
     &               gam1(i),dum2,gam10(i),alp1(i),bet1(i),
     &               pn1(i),psi1(i))
c
         do m=1,meqn
	    work(m) = qr(i,m)
            enddo
	 work(2) = qr(i,inorm)
	 work(3) = qr(i,itan)
         call ctoprm(work,rhor(i),ur(i),vr(i),dum0,dum1,pr(i),
     &               gam2(i),dum2,gam20(i),alp2(i),bet2(i),
     &               pn2(i),psi2(i))
         enddo
c
      go to (100) iriem
c
 100  continue
c     # two-shock approximation
c     #  ref. Colella & Glaz: JCP 59, 264-286 (1985)
c
      do i=imin,imax
         csql(i) = gam1(i)*rhol(i)*(pl(i)+pn1(i))+
     &             (gam1(i)-2.d0)*alp1(i)*rhol(i)**3
         csqr(i) = gam2(i)*rhor(i)*(pr(i)+pn2(i))+
     &             (gam2(i)-2.d0)*alp2(i)*rhor(i)**3 
c
         if((csql(i) .le. 0.d0) .or. (csqr(i) .le. 0.d0)) then
            write(6,*) '> Error in vrpjmp:',
     &                 ' i=',i
            write(6,*) 'rhol=',rhol(i),', pl=',pl(i),pn1(i),
     &                         gam1(i)
            write(6,*) 'rhor=',rhor(i),', pr=',pr(i),pn2(i),
     &                         gam2(i)
            stop
         endif
         enddo
c
c     # secant iteration version
      do i=imin,imax
         call rpsect(pstar(i),
     &               rhol(i),ul(i),pl(i),gam1(i),
     &               gam10(i),alp1(i),bet1(i),pn1(i),csql(i),
     &               rhor(i),ur(i),pr(i),gam2(i),gam20(i),
     &               alp2(i),bet2(i),pn2(i),csqr(i),nblow)
         if (nblow .eq. 1) stop
         enddo
c
c     # compute jumps across each wave 
      do i=imin,imax
         call wsqge(wsql,dummy,rho1,pstar(i),rhol(i),pl(i),csql(i),
     &              gam10(i),alp1(i),bet1(i),pn1(i))
         wl = dsqrt(wsql)
c
         call wsqge(wsqr,dummy,rho2,pstar(i),rhor(i),pr(i),csqr(i),
     &              gam20(i),alp2(i),bet2(i),pn2(i))
         wr = dsqrt(wsqr)
c
c        # compute mid-state velocity
         ustar = (pl(i)-pr(i)+ul(i)*wl+ur(i)*wr)/(wl+wr)
         if (dabs(ustar) .lt. 1.d-12) ustar = 0.d0
c
c        # left wave:
c        ============
         s(i,1) = ustar-wl/rho1
         call prmtoc(work,rho1,ustar,vl(i),pstar(i),
     &               gam10(i),alp1(i),bet1(i),pn1(i),psi1(i))
         do m=1,meqn
            qjump(i,m,1) = work(m)-ql(i,m)
            enddo
         qjump(i,inorm,1) = work(2)-ql(i,inorm)
         qjump(i,itan,1)  = work(3)-ql(i,itan)
c
c        # right wave:
c        =============
         s(i,3) = ustar+wr/rho2
         call prmtoc(qmid,rho2,ustar,vr(i),pstar(i),
     &               gam20(i),alp2(i),bet2(i),pn2(i),psi2(i))
         do m=1,meqn
            qjump(i,m,3) = qr(i,m)-qmid(m)
            enddo
         qjump(i,inorm,3) = qr(i,inorm)-qmid(2)
         qjump(i,itan,3)  = qr(i,itan)-qmid(3)
c
c        # slip line:
c        ============
         s(i,2) = ustar
         do m=1,meqn
            qjump(i,m,2) = qmid(m)-work(m)
            enddo
         qjump(i,inorm,2) = qmid(2)-work(2)
         qjump(i,itan,2)  = qmid(3)-work(3)
         enddo
      return
c
      end
